package movieorganizer.imdbrequester;

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import movieorganizer.userinterface.ImdbEntry;
import movieorganizer.userinterface.Movie;
import movieorganizer.userinterface.Settings;
import movieorganizer.userinterface.XMLHelper;

/** This class uses the stromboli-server for the IMDB-requests. */
public class StrombImdbRequester extends AImdbRequester {

    /** The IMDB-requester using the stromboli-server. */
    public StrombImdbRequester() {
        super();
    }

    /** Returns the inputstream for the response to the IMDB-data-request.
     * @param serverAdress the serverAdress.
     * @return the inputstream for this server-adress. */
    private InputStream getXmlStream(final String serverAdress) {
        URL xmlUrl = null;
        try {
            xmlUrl = new URL(serverAdress);
        } catch (MalformedURLException e) {
            // e.printStackTrace();
            xmlUrl = null;
        }
        InputStream result = null;
        try {
            if (xmlUrl != null) {
                result = xmlUrl.openStream();
            }
        } catch (IOException e) {
            result = null;
        }
        // TODO(mirko): dummy file. remove when server is online again.
//        try {
//            result = new FileInputStream("testMovie.xml");
//        }
//        catch (Exception e){
//
//        }

        return result;
    }

    /** Parses a XML file from a webserver containing a list of movies.
     * @param xmlStream the XML-stream.
     * @return the XML-Document created from the stream-data. */
    private Document parseMovieXmlFile(final InputStream xmlStream) {
        Document doc = null;
        DocumentBuilderFactory docFactory = DocumentBuilderFactory
                .newInstance();
        try {
            // We need a DocumentBuilder, we get it from the docFactory.
            DocumentBuilder docBuilder = docFactory.newDocumentBuilder();

            // Parse the XML-code to a DOM object.
            doc = docBuilder.parse(xmlStream);
        } catch (Exception e) {
            System.out.println(e);
        }
        return doc;
    }

    /**
     * Inserts the CDATA tag around the text <show field="..."> and ..show>.
     * @param xmlText the given XML-text.
     * @return the given text with inserted CDATA-tags.
     */
    private String insertCDATATags(final String xmlText) {
        String result = xmlText.replaceAll("\">", "\"><![CDATA[");
        result = result.replaceAll("</", "]]></");
        return result;
    }

    /**
     * Extracts the movies in DOM representation. TODO(mirko): handle receiving
     * more than one movie properly.
     * @param doc movies given in an XML-document representation.
     * @return the extracted IMDB-entries.
     */
    private ArrayList<ImdbEntry> parseMovies(final Document doc) {
        ArrayList<ImdbEntry> imdbEntries = new ArrayList<ImdbEntry>();

        // We need the root element.
        Element root = doc.getDocumentElement();

        ArrayList<String> xmlTexts = new ArrayList<String>();

        // We might receive more than one movie.
        NodeList movies = root.getElementsByTagName("hit");
        for (int i = 0; movies != null && movies.getLength() > 0
                && i < movies.getLength(); i++) {
            // Get the movie element.
            Element movie = (Element) movies.item(i);

            // Collect all strings <field show= .... </show>
            String xmlText = XMLHelper.getTextValue(movie, "title");
            xmlText = insertCDATATags(xmlText);
            xmlTexts.add(xmlText);
        }

        String validXMLresults = XMLHelper.generateXMLFromTitles(xmlTexts);
        Document validXML = XMLHelper.parseMovieXmlFile(validXMLresults);
        imdbEntries = XMLHelper.extractImdbEntries(validXML);
        return imdbEntries;
    }

    @Override
    public final ArrayList<ImdbEntry> getEntries(final Movie movie) {
        String query = cleanTitle(movie.getTitle());
        ArrayList<ImdbEntry> result = null;
        result = sendBlockRequests(query, 1);


        return result;
    }

    @Override
    public final ArrayList<ImdbEntry> sendRequest(final String query) {
        ArrayList<ImdbEntry> result = new ArrayList<ImdbEntry>();
        InputStream xmlStream = getXmlStream(Settings.SERVER_ADRESS
                + query + "*");
        if (xmlStream != null) {
            Document doc = parseMovieXmlFile(xmlStream);
            result = parseMovies(doc);
        }
        return result;
    }

}
