package movieorganizer.userinterface;

import java.awt.Dimension;

/** This class has all settings for the visual appearance of the
 * MovieOrganizer. */
public final class Settings {
    /** The standard text size. */
    public static final int TEXT_SIZE = 14;
    /** The standard font. */
    public static final String TEXT_FONT = "Arial";
    /** The cellpadding of all table-cells. */
    public static final int TABLE_CELLPADDING = 5;
    /** The row-height of all table-cells. */
    public static final int TABLE_ROWHEIGHT = 40;
    /** Minimum size of the title-column. */
    public static final int COL_TITLE_MINSIZE = 200;
    /** Size of the rating-column. */
    public static final int COL_RATING_SIZE = 70;
    /** Size of the year-column. */
    public static final int COL_YEAR_SIZE = 70;
    /** Size of the duration-column. */
    public static final int COL_DURATION_SIZE = 80;
    /** Minimum size of the director-column. */
    public static final int COL_DIRECTOR_MINSIZE = 130;
    /** Minimum size of the genres-column. */
    public static final int COL_GENRES_MINSIZE = 200;

    /** The server URL which answers the IMDB-requests. */
    public static final String SERVER_ADRESS =
         "http://stromboli.informatik.uni-freiburg.de:9999/?q=";
    /** The file where the folders and movie-information gets saved. */
    public static final String INFO_FILE = "MovieOrganizerData.XML";

    /** The default duration value. */
    public static final int DEF_DURATION = 999;
    /** The default director value. */
    public static final String DEF_DIRECTOR = "#no-data-available";
    /** The default rating value. */
    public static final float DEF_RATING = 11f;
    /** The default year value. */
    public static final int DEF_YEAR = 9999;

    /** Minimum size of the mainWindow. */
    public static final Dimension MAINWINDOW_SIZE = new Dimension(768, 400);
    /** Size of the table-scroll-pane. */
    public static final Dimension TABLEPANE_SIZE = new Dimension(687, 400);
    /** Size of the filter-pane. */
    public static final Dimension FILTERPANE_SIZE =
        new Dimension(MAINWINDOW_SIZE.width - TABLEPANE_SIZE.width,
                MAINWINDOW_SIZE.height - TABLEPANE_SIZE.height);
    /** The divider location. */
    public static final int DIVIDER_LOCATION =
        MAINWINDOW_SIZE.width - TABLEPANE_SIZE.width;

    /** All genres. Sorted lexicographically.*/
    public static final String[] ALL_GENRES = {"Action", "Comedy", "Drama",
        "Romance", "Horror", "Thriller"};

    /** This class should not be instantiated. */
    private Settings() {
    }
}
