package movieorganizer.userinterface;

import java.awt.Component;

import java.awt.Color;
import javax.swing.*;
import javax.swing.plaf.UIResource;
import javax.swing.border.Border;
import javax.swing.table.*;

/** The former DefaultTableHeaderCellRenderer from the sun-package. */
public class TableHeaderRenderer extends DefaultTableCellRenderer implements
        UIResource {
    /** The default serial-Version-UID. */
    private static final long serialVersionUID = 1L;
    /** The horizontal text position set. */
    private boolean horizontalTextPositionSet;

    /** Creates a TableHeadeRenderer element. */
    public TableHeaderRenderer() {
        setHorizontalAlignment(JLabel.CENTER);
    }

    /** Sets the horizontal text position.
     * @param textPosition the text position. */
    public final void setHorizontalTextPosition(final int textPosition) {
        horizontalTextPositionSet = true;
        super.setHorizontalTextPosition(textPosition);
    }

    /** Gets the table cell renderer component.
     * @param table the table.
     * @param value the value.
     * @param isSelected true/false.
     * @param hasFocus true/false.
     * @param row the row.
     * @param column the column.
     * @return the renderer component. */
    public Component getTableCellRendererComponent(final JTable table,
            final Object value, final boolean isSelected,
            final boolean hasFocus, final int row, final int column) {
        Icon sortIcon = null;

        boolean isPaintingForPrint = false;

        if (table != null) {
            JTableHeader header = table.getTableHeader();
            if (header != null) {
                Color fgColor = null;
                Color bgColor = null;
                if (hasFocus) {
                    fgColor = UIManager
                            .getColor("TableHeader.focusCellForeground");
                    bgColor = UIManager
                            .getColor("TableHeader.focusCellBackground");
                }
                if (fgColor == null) {
                    fgColor = header.getForeground();
                }
                if (bgColor == null) {
                    bgColor = header.getBackground();
                }
                setForeground(fgColor);
                setBackground(bgColor);

                setFont(header.getFont());

                isPaintingForPrint = header.isPaintingForPrint();
            }

            if (!isPaintingForPrint && table.getRowSorter() != null) {
                if (!horizontalTextPositionSet) {
                    // There is a row sorter, and the developer hasn't
                    // set a text position, change to leading.
                    setHorizontalTextPosition(JLabel.LEADING);
                }
                java.util.List<? extends RowSorter.SortKey> sortKeys = table
                        .getRowSorter().getSortKeys();
                if (sortKeys.size() > 0
                        && sortKeys.get(0).getColumn() == table
                                .convertColumnIndexToModel(column)) {
                    switch (sortKeys.get(0).getSortOrder()) {
                    case ASCENDING:
                        sortIcon = UIManager.getIcon("Table.ascendingSortIcon");
                        break;
                    case DESCENDING:
                        sortIcon = UIManager
                                .getIcon("Table.descendingSortIcon");
                        break;
                    case UNSORTED:
                        sortIcon = UIManager.getIcon("Table.naturalSortIcon");
                        break;
                    }
                }
            }
        }

        setText(value == null ? "" : value.toString());
        setIcon(sortIcon);

        Border border = null;
        if (hasFocus) {
            border = UIManager.getBorder("TableHeader.focusCellBorder");
        }
        if (border == null) {
            border = UIManager.getBorder("TableHeader.cellBorder");
        }
        setBorder(border);

        return this;
    }
}
