package movieorganizer.userinterface;
import java.awt.Font;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;

import javax.swing.GroupLayout;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.LayoutStyle;
import javax.swing.event.MouseInputListener;

/** The table-listener-class which handles most mouse/key-events. */
public class TableListener implements MouseInputListener, KeyListener {

    /** The movieTable is the table from the mainWindow. */
    private JTable movieTable;
    /**
     * The _tableModel is the TableModel form _movieTable. We need it to get the
     * complete movie object from the selected row.
     */
    private MovieTableModel tableModel;

    /** @param mTable the movieTable from the mainWindow.
     * @param tModel the tableModel from the movieTable. */
    public TableListener(final JTable mTable,
            final MovieTableModel tModel) {
        this.movieTable = mTable;
        this.tableModel = tModel;
    }

    @SuppressWarnings("static-access")
    @Override
    public final void mouseClicked(final MouseEvent e) {
        // A popup with the movie details should open when we make a
        // doubleclick.
        if (e.getClickCount() == 2 && e.getButton() == e.BUTTON1) {
            createPopUp();
        }
    }

    @Override
    public void mouseEntered(final MouseEvent e) {
    }

    @Override
    public void mouseExited(final MouseEvent e) {
    }

    @Override
    public void mousePressed(final MouseEvent e) {
    }

    @Override
    public void mouseReleased(final MouseEvent e) {
    }

    @Override
    public void mouseDragged(final MouseEvent e) {
    }

    @Override
    public void mouseMoved(final MouseEvent e) {
    }

    @SuppressWarnings("static-access")
    @Override
    public final void keyPressed(final KeyEvent arg0) {
        // The popup should also open if we press enter.
        if (arg0.getKeyCode() == arg0.VK_ENTER) {
            createPopUp();
        }
    }

    @Override
    public void keyReleased(final KeyEvent arg0) {
    }

    @Override
    public void keyTyped(final KeyEvent arg0) {
    }

    /** Creates the popupWindow with the movie-details.*/
    private void createPopUp() {
        // We need to know the real row on which the user clicked.
        int selectedRow = movieTable.getSelectedRow();
        selectedRow = movieTable.getRowSorter().convertRowIndexToModel(
                selectedRow);
        // We want the popup to be a new window.
        JFrame movieDetails = new JFrame();
        movieDetails.setResizable(false);
        JPanel moviePanel = new JPanel();
        movieDetails.add(moviePanel);
        // We need the complete movie object: title, rating, etc.
        Movie selectedMovie = tableModel.getMovieFromRow(selectedRow);
        movieDetails.setTitle("Details: " + selectedMovie.getTitle());
        Font font =
            new Font(Settings.TEXT_FONT, Font.PLAIN, Settings.TEXT_SIZE);

        JLabel lTitle = new JLabel("Title:");
        lTitle.setFont(font);
        JTextField tTitle = new JTextField(selectedMovie.getTitle());
        tTitle.setFont(font);
        JLabel lDirector = new JLabel("Director:");
        lDirector.setFont(font);
        JTextField tDirector = new JTextField(selectedMovie.getDirector() + "");
        tDirector.setFont(font);
        JLabel lRating = new JLabel("Rating:");
        lRating.setFont(font);
        JTextField tRating = new JTextField(selectedMovie.getRating() + "");
        tRating.setFont(font);
        JLabel lYear = new JLabel("Year:");
        lYear.setFont(font);
        JTextField tYear = new JTextField(selectedMovie.getYear() + "");
        tYear.setFont(font);
        JLabel lDuration = new JLabel("Duration:");
        lDuration.setFont(font);
        JTextField tDuration = new JTextField(selectedMovie.getDuration() + "");
        tDuration.setFont(font);
        JLabel lGenres = new JLabel("Genres:");
        lGenres.setFont(font);
        JTextField tGenres = new JTextField(selectedMovie.getGenres() + "");
        tGenres.setFont(font);
        JLabel lKeywords = new JLabel("Keywords:");
        lKeywords.setFont(font);
        JTextArea ttKeywords = new JTextArea(
                "not implemented ...");
        ttKeywords.setLineWrap(true);
        ttKeywords.setWrapStyleWord(true);
        ttKeywords.setFont(font);
        JScrollPane tKeywords = new JScrollPane(ttKeywords);

        GroupLayout mainPanelLayout = new GroupLayout(moviePanel);
        moviePanel.setLayout(mainPanelLayout);
        mainPanelLayout.setHorizontalGroup(mainPanelLayout.createParallelGroup(
                GroupLayout.Alignment.LEADING).addGroup(
                mainPanelLayout.createSequentialGroup().addContainerGap()
                        .addGroup(
                                mainPanelLayout.createParallelGroup(
                                        GroupLayout.Alignment.LEADING)
                                        .addComponent(lDirector).addComponent(
                                                lTitle).addComponent(lRating)
                                        .addComponent(lYear).addComponent(
                                                lDuration)
                                        .addComponent(lGenres).addComponent(
                                                lKeywords)).addPreferredGap(
                                LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(
                                mainPanelLayout.createParallelGroup(
                                        GroupLayout.Alignment.TRAILING, false)
                                        .addComponent(tTitle,
                                                GroupLayout.DEFAULT_SIZE, 290,
                                                Short.MAX_VALUE).addComponent(
                                                tDirector,
                                                GroupLayout.Alignment.LEADING)
                                        .addComponent(tRating).addComponent(
                                                tYear).addComponent(tDuration)
                                        .addComponent(tGenres).addComponent(
                                                tKeywords)).addContainerGap(
                                398, Short.MAX_VALUE)));
        mainPanelLayout.setVerticalGroup(mainPanelLayout.createParallelGroup(
                GroupLayout.Alignment.LEADING).addGroup(
                mainPanelLayout.createSequentialGroup().addContainerGap()
                        .addGroup(
                                mainPanelLayout.createParallelGroup(
                                        GroupLayout.Alignment.BASELINE)
                                        .addComponent(lTitle).addComponent(
                                                tTitle,
                                                GroupLayout.PREFERRED_SIZE,
                                                GroupLayout.DEFAULT_SIZE,
                                                GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(
                                LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(
                                mainPanelLayout.createParallelGroup(
                                        GroupLayout.Alignment.BASELINE)
                                        .addComponent(lDirector).addComponent(
                                                tDirector,
                                                GroupLayout.PREFERRED_SIZE,
                                                GroupLayout.DEFAULT_SIZE,
                                                GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(
                                LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(
                                mainPanelLayout.createParallelGroup(
                                        GroupLayout.Alignment.BASELINE)
                                        .addComponent(lRating).addComponent(
                                                tRating,
                                                GroupLayout.PREFERRED_SIZE,
                                                GroupLayout.DEFAULT_SIZE,
                                                GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(
                                LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(
                                mainPanelLayout.createParallelGroup(
                                        GroupLayout.Alignment.BASELINE)
                                        .addComponent(lYear).addComponent(
                                                tYear,
                                                GroupLayout.PREFERRED_SIZE,
                                                GroupLayout.DEFAULT_SIZE,
                                                GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(
                                LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(
                                mainPanelLayout.createParallelGroup(
                                        GroupLayout.Alignment.BASELINE)
                                        .addComponent(lDuration).addComponent(
                                                tDuration,
                                                GroupLayout.PREFERRED_SIZE,
                                                GroupLayout.DEFAULT_SIZE,
                                                GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(
                                LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(
                                mainPanelLayout.createParallelGroup(
                                        GroupLayout.Alignment.BASELINE)
                                        .addComponent(lGenres).addComponent(
                                                tGenres,
                                                GroupLayout.PREFERRED_SIZE,
                                                GroupLayout.DEFAULT_SIZE,
                                                GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(
                                LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(
                                mainPanelLayout.createParallelGroup(
                                        GroupLayout.Alignment.BASELINE)
                                        .addComponent(lKeywords).addComponent(
                                                tKeywords, 50, 50, 100))
                        .addContainerGap(207, Short.MAX_VALUE)));
        movieDetails.setSize(400, 300);
        movieDetails.setLocation(600, 0);
        movieDetails.setVisible(true);
    }

}
