package movieorganizer.userinterface;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;

import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTable;

/**
 * This class is used to handle the RIGHT-click pop up menu. The user should be
 * able to choose from the imdb-entries. He also should be able to enter an own
 * title.
 */
public class TablePopUpListener extends MouseAdapter {
    /** The JTable from the main window. */
    private JTable movieTable;
    /** The tableModel from movieTable. */
    private MovieTableModel tableModel;
    /** The PopUpMenu which we want to show. */
    private JPopupMenu popUpMenu;
    /** The sub menu for the imdb entries. */
    private JMenu imdbEntriesMenu;
    /** The main window with the movieTable. */
    private MainWindow mainWindow;

    /** @param mTable the table from the mainWindow.
     * @param tModel the tableModel from the movieTable.
     * @param pMenu the popUpMenu.
     * @param iMenu the imdbEntriesMenu.
     * @param mWindow the mainWindow. */
    public TablePopUpListener(final JTable mTable,
            final MovieTableModel tModel, final JPopupMenu pMenu,
            final JMenu iMenu, final MainWindow mWindow) {
        this.movieTable = mTable;
        this.tableModel = tModel;
        this.popUpMenu = pMenu;
        this.imdbEntriesMenu = iMenu;
        this.mainWindow = mWindow;
    }

    @SuppressWarnings("static-access")
    @Override
    public final void mouseReleased(final MouseEvent arg0) {
        if (arg0.getButton() == arg0.BUTTON2
                || arg0.getButton() == arg0.BUTTON3) {
            // Determine the selected row.
            popUpMenu.show(arg0.getComponent(), arg0.getX(), arg0.getY());
            int selectedRow = movieTable.rowAtPoint(arg0.getPoint());
            movieTable.setRowSelectionInterval(selectedRow, selectedRow);
            // Change the correct row in the model.
            selectedRow = movieTable.getRowSorter().convertRowIndexToModel(
                    selectedRow);

            // Get the selected movie.
            final Movie selectedMovie = tableModel.getMovieFromRow(selectedRow);
            final ArrayList<ImdbEntry> imdbEntries = selectedMovie
                    .getImdbEntries();

            // Clear the menu before adding the new entries.
            imdbEntriesMenu.removeAll();

            // Add the imdb entries of the selected movie.
            for (int i = 0; i < imdbEntries.size(); i++) {
                final ImdbEntry selectedImdbEntry = imdbEntries.get(i);
                String imdbEntryTitle = imdbEntries.get(i).getTitle();
                JMenuItem menuItem = new JMenuItem(imdbEntryTitle);
                // When an entry gets selected, the table should be refreshed.
                menuItem.addActionListener(new ActionListener() {
                    @Override
                    public void actionPerformed(final ActionEvent arg0) {
                        // Change to the selected imdb entry.
                        changeImdbEntryTo(selectedImdbEntry);
                        // Update the table.
                        mainWindow.updateTable(MovieManager.getMovies());
                        // Write the updated title to the XML.
                        MovieManager.writeMoviesAsXml();
                    }
                });
                imdbEntriesMenu.add(menuItem);
            }
        }
    }

    /** Changes the imdb-entry from the selected row to a given imdbEntry.
     * @param imdbEntry the IMDB-entry. */
    private void changeImdbEntryTo(final ImdbEntry imdbEntry) {
        // This is row on which we clicked.
        int selectedRow = movieTable.getSelectedRow();
        // But we need the to change the real row, the one from the model.
        selectedRow = movieTable.getRowSorter().convertRowIndexToModel(
                selectedRow);
        tableModel.getMovieFromRow(selectedRow).setTitle(imdbEntry.getTitle());
        tableModel.getMovieFromRow(selectedRow).setDirector(
                imdbEntry.getDirector());
        tableModel.getMovieFromRow(selectedRow).setDuration(
                imdbEntry.getDuration());
        tableModel.getMovieFromRow(selectedRow)
                .setGenres(imdbEntry.getGenres());
        tableModel.getMovieFromRow(selectedRow)
                .setRating(imdbEntry.getRating());
        tableModel.getMovieFromRow(selectedRow).setYear(imdbEntry.getYear());
    }
}
